/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * Portions of this software are based upon public domain software
 * originally written at the National Center for Supercomputing Applications,
 * University of Illinois, Urbana-Champaign.
 */

#include "ap_config.h"
#include "httpd.h"
#include "ap_iol.h"
#include <malloc.h>


typedef struct {
    ap_iol iol;
    ap_file_t *file;
} iol_file;

#define method(syscall, args)	\
    static ap_status_t file_##syscall args \
    { \
        ap_status_t rv; \
	iol_file *iol = (iol_file *)viol; \
	*nbytes = arg2; \
        /* ignore partial reads and writes, the upper layer has to handle \
           them anyhow, so we can return early */ \
	do { \
	    rv = syscall(iol->file, (void*)arg1, nbytes); \
	} while (rv == APR_EINTR); \
        if (rv != APR_SUCCESS) { \
            *nbytes = 0; \
        } \
	return rv; \
    }

method(ap_write, (ap_iol *viol, const char *arg1, ap_size_t arg2, ap_ssize_t *nbytes))
method(ap_read, (ap_iol *viol, char *arg1, ap_size_t arg2, ap_ssize_t *nbytes))

static ap_status_t file_ap_writev(ap_iol *viol, const struct iovec *vec, ap_size_t nvec, 
                                  ap_ssize_t *nbytes) 
{
    ap_status_t rv;
    iol_file *iol = (iol_file *) viol;
    do {
        rv = ap_writev(iol->file, vec, nvec, nbytes);
    } while (rv == APR_EINTR);
    if (rv != APR_SUCCESS) {
        *nbytes = 0;
    }
    return rv;
}


/* This function will clean-up the iol struct and close the file... */
static ap_status_t file_close(ap_iol *viol)
{
    ap_status_t rv;
    iol_file *iol = (iol_file *)viol;
    rv = ap_close(iol->file);
    free(iol);

    return rv;
}

static ap_status_t file_setopt(ap_iol *viol, ap_iol_option opt,
                               const void *value)
{
    iol_file *iol = (iol_file *)viol;

    switch (opt) {
    case AP_IOL_TIMEOUT:
        return ap_set_pipe_timeout(iol->file, *(const int*)value);
    default:
        return APR_EINVAL;
    }
}

static ap_status_t file_getopt(ap_iol *viol, ap_iol_option opt, void *value)
{
    return APR_EINVAL;
}

static const ap_iol_methods file_methods = {
    file_close,
    file_ap_write,
    file_ap_writev,
    file_ap_read,
    file_setopt,
    file_getopt
};

/*
 * Create an ap_iol struct given an apr_file_t
 */
ap_iol *ap_create_file_iol(ap_file_t *file)
{
    iol_file *iol;
    iol = malloc(sizeof(iol_file));
    iol->iol.methods = &file_methods;
    iol->file = file;
    return (ap_iol *)iol;
}

