/* This mutex implementation courtesy of Christopher Tate (ctate@be.com) */

// BeOS mutex implementation

#include "mutex.h"
#include <kernel/OS.h>
#include <stdio.h>

// mutex operations
static void lazy_init_mutex(be_mutex_t* mutex)
{
	int32 v = atomic_or(&mutex->init, 1);
	if (2000 == v)			// we're the first, so do the init
	{
		be_mutex_init(mutex, NULL);
	}
	else							// we're not the first, so wait until the init is finished
	{
		while (mutex->init != 9999) snooze(10000);
	}
}

status_t be_mutex_init(be_mutex_t* mutex, be_mutexattr_t* attr)
{
	if (!mutex) return B_BAD_VALUE;
	if (mutex->init == 9999) return EALREADY;

	mutex->lock = create_sem(1, "BeMutex");
	mutex->owner = -1;
	mutex->init = 9999;
	return B_OK;
}

status_t be_mutex_destroy(be_mutex_t* mutex)
{
	if (!mutex) return B_BAD_VALUE;
	if (mutex->init != 9999) return B_NO_INIT;
	delete_sem(mutex->lock);
	return B_OK;
}

status_t be_mutex_lock(be_mutex_t* mutex)
{
	status_t err;

	if (!mutex) return B_BAD_VALUE;
	if (mutex->init < 2000) return B_NO_INIT;
	lazy_init_mutex(mutex);

	err = acquire_sem(mutex->lock);
	if (!err) mutex->owner = find_thread(NULL);
	return err;
}

status_t be_mutex_unlock(be_mutex_t* mutex)
{
	if (!mutex) return B_BAD_VALUE;
	if (mutex->init < 2000) return B_NO_INIT;
	lazy_init_mutex(mutex);

	if (mutex->owner != find_thread(NULL)) return ENOLCK;
	mutex->owner = -1;
	release_sem(mutex->lock);
	return B_OK;
}
